Package: multiHiCcompare
Title: Normalize and detect differences between Hi-C datasets when
        replicates of each experimental condition are available
Version: 1.30.0
Authors@R: c(
    person(given = "Mikhail",
           family = "Dozmorov",
           role = c("aut", "cre"),
           email = "mikhail.dozmorov@gmail.com",
           comment = c(ORCID = "0000-0002-0086-8358")),
    person(given = "John",
           family = "Stansfield",
           role = c("aut"),
           email = "stansfieldjc@vcu.edu"))
Description: multiHiCcompare provides functions for joint normalization 
    and difference detection in multiple Hi-C datasets. This extension of
    the original HiCcompare package now allows for Hi-C experiments with 
    more than 2 groups and multiple samples per group. multiHiCcompare 
    operates on processed Hi-C data in the form of sparse upper triangular 
    matrices. It accepts four column (chromosome, region1, region2, IF)
    tab-separated text files storing chromatin interaction matrices. 
    multiHiCcompare provides cyclic loess and fast loess (fastlo) methods
    adapted to jointly normalizing Hi-C data. Additionally, it provides 
    a general linear model (GLM) framework adapting the edgeR package to detect
    differences in Hi-C data in a distance dependent manner. 
Depends: R (>= 4.0.0)
Imports: data.table, dplyr, HiCcompare, edgeR, BiocParallel, qqman,
        pheatmap, methods, GenomicRanges, graphics, stats, utils,
        pbapply, GenomeInfoDbData, GenomeInfoDb, aggregation
biocViews: Software, HiC, Sequencing, Normalization
License: MIT + file LICENSE
Encoding: UTF-8
LazyData: true
Suggests: knitr, rmarkdown, testthat, BiocStyle
RoxygenNote: 7.1.1
VignetteBuilder: knitr
BugReports: https://github.com/dozmorovlab/multiHiCcompare/issues
URL: https://github.com/dozmorovlab/multiHiCcompare
git_url: https://git.bioconductor.org/packages/multiHiCcompare
git_branch: RELEASE_3_23
git_last_commit: 0adf2ca
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.23
Date/Publication: 2026-04-28
NeedsCompilation: no
Packaged: 2026-04-28 23:30:33 UTC; biocbuild
Author: Mikhail Dozmorov [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-0086-8358>),
  John Stansfield [aut]
Maintainer: Mikhail Dozmorov <mikhail.dozmorov@gmail.com>
