#' A 4 column sparse matrix for a Hi-C matrix.
#'
#' A matrix object with 4 columns and 54307 rows. 
#'     Sample 4 of 7 included as example Hi-C data.
#'     This is replicate 4 of HCT-116 cell
#'     line at 100KB resolution from chHCT116_r22.
#'
#' @source Data from Rao 2017. See their website at
#'     \url{http://www.aidenlab.org/}
#'      Or the the GEO link to download the data
#'     \url{https://www.ncbi.nlm.nih.gov/geo/query/acc.cgi?acc=GSE104888}
#'
#' @return A matrix
"HCT116_r4"