#' hicexp object with 4 samples from two groups.  
#'
#' A hicexp object with a hic_table slot containing 666 rows
#'     from chromosome 22 at 1MB resolution.
#'
#' @source Data from Rao 2017. See their website at
#'     \url{http://www.aidenlab.org/}
#'      Or the the GEO link to download the data
#'     \url{https://www.ncbi.nlm.nih.gov/geo/query/acc.cgi?acc=GSE104888}
#'
#' @return A hicexp object
#' 
"hicexp2"